# mse2.r
# This R program creates the simulation results reported in the paper
# "Model Averaging, Asymtpotic Risk, and Regressor Groups"
# by Bruce E. Hansen, Department of Economics, University of Wisconsin
# http://www.ssc.wisc.edu

# The program uses the packages glmnet and quadprog
# This program produces the robustness checks

library(glmnet)
library(quadprog)

rep=2000
mn=5;

rs=seq(0,.9,.05)
cn=length(rs)
ns=c(50,150,500,1000)

nn=length(ns)
aa=c(0,1,2,3)
an=length(aa)

amse=array(0,dim=c(mn,an,nn,cn,7))

for (mi in 2:mn){

if (mi==4) rho=.5 else rho=0
if (mi==5) k=24 else k=12

for (ai in 1:an) {
a=aa[ai]
gam=seq(1,k,1)^(-a)
gg=(t(gam)%*%gam)*(1-rho)+(sum(gam)^2)%*%rho
cs=sqrt(rs/(1-rs))/gg

for (ni in 1:nn) {
n=ns[ni]

for (ci in 1:cn) {

c=cs[ci]
beta=gam*c
beta0=rbind(0,as.matrix(beta))

store0=matrix(0,rep,1)
store1=matrix(0,rep,7)

for (i in 1:rep) {

x=matrix(rnorm(n*k),n,k)*sqrt(1-rho)+matrix(rnorm(n),n,k)*sqrt(rho)

if (mi==3) sd=as.matrix(sqrt((1+x[,1]^2)/2)) else sd=matrix(1,n,1)
e=matrix(rnorm(n))*sd
if (mi==2) e=(4/3)-(matrix(runif(n,0,1)) <= .8)*(5/3)+e*sqrt(5/9)

y=x%*%beta+e

# Least-Squares 

ols=lm(y~x)
bols=as.matrix(ols$coef)
ehat=as.matrix(ols$residuals)
sig=sum(ehat^2)/(n-k-1)
store0[i,1]=sum((bols-beta0)^2)

# Intercept Model

mu=mean(y)
b0=rbind(mu,matrix(0,k,1))
e0=y-mu
sig0=mean(e0^2)

# Correct Ordering 

bbeta=matrix(0,k+1,k+1)
aic=matrix(0,k+1,1)
bic=matrix(0,k+1,1)
ee=matrix(0,n,k+1)
  bbeta[,1]=b0
  ee[,1]=e0
  aic[1]=n*log(sig0)+2
  bic[1]=n*log(sig0)
for (j in 1:k)
  {
  olsj=lm(y~x[,1:j])
  bbeta[1:(1+j),1+j]=as.matrix(olsj$coef)
  ej=olsj$residuals
  ee[,1+j]=ej
  sigj=mean(ej^2)
  aic[1+j]=n*log(sigj)+2*(1+j)
  bic[1+j]=n*log(sigj)+log(n)*(1+j)
  }
# Grouped
s=seq(1,k+1,4)
sbeta=bbeta[,s]
see=ee[,s]

# MMA4
m=ncol(see)
a1=t(see)%*%see
if (qr(a1)$rank<m) a1=a1+diag(m)*1e-10
a2=as.matrix(-s*sig)
a3=t(rbind(matrix(1,1,m),diag(m),-diag(m)))
a4=rbind(1,matrix(0,m,1),matrix(-1,m,1))
QP=solve.QP(a1,a2,a3,a4,1)
w=QP$solution
bmma4=sbeta%*%w
store1[i,1]=sum((bmma4-beta0)^2)

# MMA
m=ncol(ee)
a1=t(ee)%*%ee
if (qr(a1)$rank<m) a1=a1+diag(m)*1e-10
a2=as.matrix(-seq(1,k+1,1)*sig)
a3=t(rbind(matrix(1,1,m),diag(m),-diag(m)))
a4=rbind(1,matrix(0,m,1),matrix(-1,m,1))
QP=solve.QP(a1,a2,a3,a4,1)
w=QP$solution
bmma=bbeta%*%w
store1[i,2]=sum((bmma-beta0)^2)

# Stein

bb=bols-b0
xx=cbind(matrix(1,n,1),x)%*%bb
w=sig*(k-2)/(t(xx)%*%xx)
if (w>1) w=1
bstein=bols-bb%*%w
store1[i,3]=sum((bstein-beta0)^2)

# Lasso

fit <- cv.glmnet(x,y,nfolds=5,alpha=1,type.measure="mse")
blasso <- as.matrix(coef(fit,s=fit$lambda.min))
store1[i,4]=sum((blasso-beta0)^2)

# Smoothed BIC
w=exp(-bic/2)
bma=bbeta%*%(w/sum(w))
store1[i,5]=sum((bma-beta0)^2)

# Smoothed AIC
w=exp(-aic/2)
baic=bbeta%*%(w/sum(w))
store1[i,6]=sum((baic-beta0)^2)

# Reversed Ordering 
r=seq(k,1,-1)
xr=x[,r]
betar=rbind(0,as.matrix(beta[r]))

bbeta=matrix(0,k+1,k+1)
ee=matrix(0,n,k+1)
  bbeta[,1]=b0
  ee[,1]=e0
for (j in 1:k)
  {
  olsj=lm(y~xr[,1:j])
  bbeta[1:(1+j),1+j]=as.matrix(olsj$coef)
  ej=olsj$residuals
  ee[,1+j]=ej
  }
sbeta=bbeta[,s]
see=ee[,s]
m=ncol(see)
a1=t(see)%*%see
if (qr(a1)$rank<m) a1=a1+diag(m)*1e-10
a2=as.matrix(-s*sig)
a3=t(rbind(matrix(1,1,m),diag(m),-diag(m)))
a4=rbind(1,matrix(0,m,1),matrix(-1,m,1))
QP=solve.QP(a1,a2,a3,a4,1)
w=QP$solution
bmmar=sbeta%*%w
store1[i,7]=sum((bmmar-betar)^2)

} # end loop over i

mse0=mean(store0)
mse1=colMeans(store1)/mse0
amse[mi,ai,ni,ci,]=mse1

} # end loop over ci
} # end loop over ni

mse0=matrix(1,cn,1)

lthick2=matrix(2,8,1)
col2=c(1,2,3,4,6,3,2,1)
ltype2=c(1,5,2,4,6,3,3,3)
ylim1=c(0,1.5)

windows()
layout(matrix(c(1,2,3,4,5,5), ncol=2, byrow=TRUE), heights=c(4,4,1))
for (ni in 1:nn) {
mse2=cbind(amse[mi,ai,ni,,],mse0)
main2=paste("model=",mi,", n=",ns[ni],", a=",aa[ai],sep="")
par(mai=rep(0.5, 4))
matplot(rs,mse2,type="l",main=main2,xlab="R-squared",ylab="MSE",lty=ltype2,lwd=lthick2,ylim=ylim1,col=col2)
}
par(mai=c(0,0,0,0))
plot.new()
legend(x="center",ncol=4,legend=c(expression("MMA"[4]),"MMA","Stein","Lasso","BMA","SAIC","Reversed","OLS"),lwd=lthick2,lty=ltype2,bty="n",col=col2,title="Estimators")
savePlot(file=paste("model",mi,".",ai,".pdf",sep=""),type="pdf")

windows()
layout(matrix(c(1,2,3,4,5,5), ncol=2, byrow=TRUE), heights=c(4,4,1))
for (ni in 1:nn) {
mse2=cbind(amse[mi,ai,ni,,],mse0)
par(mai=rep(0.5, 4))
main2=paste("n=",ns[ni],sep="")
matplot(rs,mse2,type="l",main=main2,xlab="R-squared",ylab="MSE",lty=ltype2,lwd=lthick2,ylim=ylim1,col=col2)
}
par(mai=c(0,0,0,0))
plot.new()
legend(x="center",ncol=4,legend=c(expression("MMA"[4]),"MMA","Stein","Lasso","BMA","SAIC","Reversed","OLS"),lwd=lthick2,lty=ltype2,bty="n",col=col2,title="Estimators")
savePlot(file=paste("model",mi,".",ai,".eps",sep=""),type="eps")

save.image(file="mse2")

} # end loop over ai
} # end loop over mi