# mse3.r
# This R program creates the simulation results reported in the paper
# "Model Averaging, Asymtpotic Risk, and Regressor Groups"
# by Bruce E. Hansen, Department of Economics, University of Wisconsin
# http://www.ssc.wisc.edu

# The program uses the packages glmnet and quadprog
# This program produces the time series results. 


library(glmnet)
library(quadprog)

rep=10000
k=12

rs=seq(0,.95,.05)
cn=length(rs)
ns=c(50,150,500,1000)

nn=length(ns)
amse=array(0,dim=c(nn,cn,6))

for (ni in 1:nn) {
n=ns[ni]

for (ci in 1:cn) {

theta=rs[ci]
beta=1-(1:k)/(k+1)
beta=beta*theta/sum(beta)
beta0=rbind(0,as.matrix(beta))

store0=matrix(0,rep,1)
store1=matrix(0,rep,6)

for (i in 1:rep) {

yy=arima.sim(n=(n+k),list(ar=beta))
y=yy[(k+1):(k+n)]
x=yy[(k):(k+n-1)]
for (j in 2:k) {
  x = cbind(x,yy[(k+1-j):(k+n-j)])
}

# Least-Squares 

ols=lm(y~x)
bols=as.matrix(ols$coef)
ehat=as.matrix(ols$residuals)
sig=sum(ehat^2)/(n-k-1)
store0[i,1]=sum((bols-beta0)^2)

# Intercept Model

mu=mean(y)
b0=rbind(mu,matrix(0,k,1))
e0=y-mu
sig0=mean(e0^2)

# Correct Ordering 

bbeta=matrix(0,k+1,k+1)
aic=matrix(0,k+1,1)
bic=matrix(0,k+1,1)
ee=matrix(0,n,k+1)
  bbeta[,1]=b0
  ee[,1]=e0
  aic[1]=n*log(sig0)+2
  bic[1]=n*log(sig0)
for (j in 1:k)
  {
  olsj=lm(y~x[,1:j])
  bbeta[1:(1+j),1+j]=as.matrix(olsj$coef)
  ej=olsj$residuals
  ee[,1+j]=ej
  sigj=mean(ej^2)
  aic[1+j]=n*log(sigj)+2*(1+j)
  bic[1+j]=n*log(sigj)+log(n)*(1+j)
  }
# MMA4
s=seq(1,k+1,4)
sbeta=bbeta[,s]
see=ee[,s]
m=ncol(see)
a1=t(see)%*%see
if (qr(a1)$rank<m) a1=a1+diag(m)*1e-10
a2=as.matrix(-s*sig)
a3=t(rbind(matrix(1,1,m),diag(m),-diag(m)))
a4=rbind(1,matrix(0,m,1),matrix(-1,m,1))
QP=solve.QP(a1,a2,a3,a4,1)
w=QP$solution
bmma4=sbeta%*%w
store1[i,1]=sum((bmma4-beta0)^2)

# MMA
m=ncol(ee)
a1=t(ee)%*%ee
if (qr(a1)$rank<m) a1=a1+diag(m)*1e-10
a2=as.matrix(-seq(1,k+1,1)*sig)
a3=t(rbind(matrix(1,1,m),diag(m),-diag(m)))
a4=rbind(1,matrix(0,m,1),matrix(-1,m,1))
QP=solve.QP(a1,a2,a3,a4,1)
w=QP$solution
bmma=bbeta%*%w
store1[i,2]=sum((bmma-beta0)^2)

# Stein

bb=bols-b0
xx=cbind(matrix(1,n,1),x)%*%bb
w=sig*(k-2)/(t(xx)%*%xx)
if (w>1) w=1
bstein=bols-bb%*%w
store1[i,3]=sum((bstein-beta0)^2)

# Lasso

fit <- cv.glmnet(x,y,nfolds=5,alpha=1,type.measure="mse")
blasso <- as.matrix(coef(fit,s=fit$lambda.min))
store1[i,4]=sum((blasso-beta0)^2)

# Smoothed BIC
w=exp(-bic/2)
bma=bbeta%*%(w/sum(w))
store1[i,5]=sum((bma-beta0)^2)

# Smoothed AIC
w=exp(-aic/2)
baic=bbeta%*%(w/sum(w))
store1[i,6]=sum((baic-beta0)^2)

} # end loop over i

mse0=mean(store0)
mse1=colMeans(store1)/mse0
amse[ni,ci,]=mse1

} # end loop over ci
} # end loop over ni

mse0=matrix(1,cn,1)

lthick1=matrix(2,6,1)
col1=c(1,2,3,4,6,1)
ltype1=c(1,5,2,4,6,3)
ylim1=c(0,1.5)

layout(matrix(c(1,2,3,4,5,5), ncol=2, byrow=TRUE), heights=c(4,4,1))
for (ni in 1:nn) {
mse1=cbind(amse[ni,,(1:5)],mse0)
main1=paste("n=",ns[ni],sep="")
par(mai=rep(0.5, 4))
matplot(rs,mse1,type="l",main=main1,xlab=expression(theta),ylab="MSE",lty=ltype1,lwd=lthick1,ylim=ylim1,col=col1)
}
par(mai=c(0,0,0,0))
plot.new()
legend(x="center",ncol=3,legend=c(expression("MMA"[4]),"MMA","Stein","Lasso","BMA","OLS"),lwd=lthick1,lty=ltype1,bty="n",col=col1,title="Estimators")
savePlot(file="fig5",type="pdf")
savePlot(file="fig5",type="eps")

lthick2=matrix(2,7,1)
col2=c(1,2,3,4,6,3,1)
ltype2=c(1,5,2,4,6,3,3)
windows()
layout(matrix(c(1,2,3,4,5,5), ncol=2, byrow=TRUE), heights=c(4,4,1))
for (ni in 1:nn) {
mse2=cbind(amse[ni,,],mse0)
main2=paste("n=",ns[ni],sep="")
par(mai=rep(0.5, 4))
matplot(rs,mse2,type="l",main=main2,xlab="R-squared",ylab="MSE",lty=ltype2,lwd=lthick2,ylim=ylim1,col=col2)
}
par(mai=c(0,0,0,0))
plot.new()
legend(x="center",ncol=4,legend=c(expression("MMA"[4]),"MMA","Stein","Lasso","BMA","SAIC","OLS"),lwd=lthick2,lty=ltype2,bty="n",col=col2,title="Estimators")
savePlot(file="time",type="pdf")
savePlot(file="time",type="eps")

save.image(file="mse3")

